@@ -1,7 +1,7 @@
 """Manage software dependencies"""
 from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
 from . import config, defs, util
-import sys, os, subprocess, logging, glob
+import sys, os, subprocess, logging, glob, json
 from shutil import which
 from pprint import pformat
 import platform, distro, requests
@@ -62,9 +62,11 @@ def _check_pkg_repo():
                                logger=logger)
         found = (res.returncode == 0)
     elif (which("dnf")):
-        pkgs  = util.run_and_log(["dnf", "copr", "list", "--enabled"],
-                                 logger=logger,
-                                 output=True)
+        pkgs  = util.run_and_log(
+            util.root_cmd(["dnf", "copr", "list", "--enabled"]),
+            logger=logger,
+            output=True
+        )
         found =  ("copr.fedorainfracloud.org/blockstream/satellite" in pkgs)
     elif (which("yum")):
         yum_sources = glob.glob("/etc/yum.repos.d/*")
@@ -116,31 +118,25 @@ def _enable_pkg_repo(interactive, dry):
 
 
 def _update_pkg_repo(interactive, dry):
-    """Update binary package repository"""
-    cmds = list()
-    if (which("apt")):
-        cmd = ["apt", "update"]
-        if (not interactive):
-            cmd.append("-y")
-        cmds.append(cmd)
-    elif (which("dnf")):
-        cmd = ["dnf", "check-update"]
-        if (not interactive):
-            cmd.append("-y")
-        cmds.append(cmd)
-    elif (which("yum")):
-        cmd = ["yum", "check-update"]
-        if (not interactive):
-            cmd.append("-y")
-        cmds.append(cmd)
-    else:
-        raise RuntimeError("Could not find a supported package manager")
+    """Update APT's package index
 
-    for cmd in cmds:
-        if (dry):
-            print(" ".join(util.root_cmd(cmd)))
-        else:
-            util.run_and_log(util.root_cmd(cmd), logger=logger)
+    NOTE: this function updates APT only. On dnf/yum, there is no need to run an
+    update manually. The tool (dnf/yum) updates the package index automatically
+    when an install/upgrade command is called.
+
+    """
+
+    if (not which("apt")):
+        return
+
+    cmd = ["apt", "update"]
+    if (not interactive):
+        cmd.append("-y")
+
+    if (dry):
+        print(" ".join(util.root_cmd(cmd)))
+    else:
+        util.run_and_log(util.root_cmd(cmd), logger=logger)
 
 
 def _install_packages(apt_list, dnf_list, yum_list, interactive=True,
@@ -208,7 +204,7 @@ def _install_common(interactive=True, update=False, dry=False, btc=False):
                       update, dry)
 
     # Enable our binary package repository
-    if not _check_pkg_repo():
+    if dry or (not _check_pkg_repo()):
         _enable_pkg_repo(interactive, dry)
 
     # Install bitcoin-satellite
@@ -217,8 +213,20 @@ def _install_common(interactive=True, update=False, dry=False, btc=False):
                         "bitcoin-satellite-tx"]
         dnf_pkg_list = ["bitcoin-satellite", "bitcoin-satellite-qt"]
         yum_pkg_list = ["bitcoin-satellite", "bitcoin-satellite-qt"]
-        _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list, interactive,
-                          update, dry)
+
+        if (which("dnf")):
+            # dnf matches partial package names, and so it seems that when
+            # bitcoin-satellite and bitcoin-satellite-qt are installed in one
+            # go, only the latter gets installed. The former is matched to
+            # bitcoin-satellite-qt and assumed as installed. Hence, as a
+            # workaround, install the two packages separately.
+            for pkg in dnf_pkg_list:
+                _install_packages(apt_pkg_list, [pkg], [pkg],
+                                  interactive, update, dry)
+        else:
+            _install_packages(apt_pkg_list, [], yum_pkg_list,
+                              interactive, update, dry)
+
 
 
 def _install_sdr(interactive=True, update=False, dry=False):
@@ -256,6 +264,58 @@ def _install_standalone(interactive=True, update=False, dry=False):
                       update, dry)
 
 
+def _check_cli_updates(interactive):
+    """Check if the CLI has updates"""
+
+    logger.debug("Checking blocksat-cli updates")
+
+    # If pip is not available, then definitely the CLI was not installed via pip
+    if (not which("pip3")):
+        return
+
+    # Is blocksat-cli installed?
+    res = subprocess.run(["pip3", "show", "blocksat-cli"],
+                         stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+    found = (res.returncode == 0)
+
+    # If the CLI was not installed via pip, don't check updates
+    if (not found):
+        return
+
+    # Is the CLI outdated?
+    try:
+        outdated_list = json.loads(
+            subprocess.check_output(["pip3", "list", "--outdated",
+                                     "--format", "json"],
+                                    stderr=subprocess.DEVNULL).decode()
+        )
+    except subprocess.CalledProcessError:
+        # Don't break if the command fails. It could be a problem on pip. For
+        # example, there is a bug in which pip throws a TypeError on the "list
+        # --outdated" command saying: '>' not supported between instances of
+        # 'Version' and 'Version'
+        logger.warning("Failed to check blocksat-cli updates.")
+        return
+
+    cli_outdated = False
+    for package in outdated_list:
+        if (package['name'] == "blocksat-cli"):
+            cli_outdated = True
+            print("Update available for blocksat-cli")
+            print("Current version: {}.\nLatest version: {}.".format(
+                package['version'], package['latest_version']
+            ))
+            if ((not interactive) or util._ask_yes_or_no("Update now?")):
+                cmd = ["pip3", "install", "blocksat-cli", "--upgrade"]
+                util.run_and_log(util.root_cmd(cmd))
+                print("Updated successfully")
+                print("Please, run command again to use the new version")
+                exit()
+
+    if (not cli_outdated):
+        logger.debug("blocksat-cli is up-to-date")
+
+
 def _print_help(args):
     """Re-create argparse's help menu"""
     parser     = ArgumentParser()
@@ -298,7 +358,7 @@ def subparser(subparsers):
                     help="Install bitcoin-satellite")
     p1.set_defaults(func=run, update=False)
 
-    p2 = subsubp.add_parser('update',
+    p2 = subsubp.add_parser('update', aliases=['upgrade'],
                             description="Update software dependencies",
                             help='Update software dependencies')
     p2.add_argument("--target",
@@ -321,6 +381,14 @@ def subparser(subparsers):
 
 def run(args):
     """Run installations"""
+
+    # Interactive installation? I.e., requires user to press "y/n"
+    interactive = (not args.yes)
+
+    # Check CLI updates
+    if (not args.dry_run):
+        _check_cli_updates(interactive)
+
     if (args.target is not None):
         target_map = {
             "sdr"        : defs.sdr_setup_type,
@@ -338,8 +406,11 @@ def run(args):
         util.fill_print("Some commands require root access and will prompt "
                         "for password")
 
-    # Interactive installation? I.e., requires user to press "y/n"
-    interactive = (not args.yes)
+    if (args.dry_run):
+        util._print_header("Dry Run Mode")
+
+    # Update package index
+    _update_pkg_repo(interactive, args.dry_run)
 
     # Common dependencies (regardless of setup)
     _install_common(interactive=interactive, update=args.update,
@@ -362,12 +433,19 @@ def run(args):
 
 
 def drivers(args):
+    # Interactive installation? I.e., requires user to press "y/n"
+    interactive = (not args.yes)
+
+    # Check CLI updates
+    if (not args.dry_run):
+        _check_cli_updates(interactive)
+
     if (os.geteuid() != 0 and not args.dry_run):
         util.fill_print("Some commands require root access and will prompt "
                         "for password")
 
-    # Interactive installation? I.e., requires user to press "y/n"
-    interactive = (not args.yes)
+    if (args.dry_run):
+        util._print_header("Dry Run Mode")
 
     runner = util.ProcessRunner(logger, args.dry_run)
 
@@ -425,7 +503,7 @@ def drivers(args):
         else:
             logger.warning("Could not find an available kernel update")
 
-    #_update_pkg_repo(interactive, args.dry_run)
+    _update_pkg_repo(interactive, args.dry_run)
     _install_packages(apt_pkg_list, dnf_pkg_list, yum_pkg_list,
                       interactive=interactive, update=False, dry=args.dry_run)
 
@@ -489,7 +567,8 @@ def drivers(args):
     runner.run(util.root_cmd(["tar", "jxvf", fw_tarball, "-C",
                               "/lib/firmware/"]), cwd = driver_src_dir)
 
-    print("Installation completed successfully. Please reboot now.")
+    if (not args.dry_run):
+        print("Installation completed successfully. Please reboot now.")
 
 
 def check_apps(apps):
